@@ -161,12 +161,9 @@ test_bit(int nr, unsigned long addr)
 #define isSwapBacked(flags)	test_bit(NUMBER(PG_swapbacked), flags)
 #define isHWPOISON(flags)	(test_bit(NUMBER(PG_hwpoison), flags) \
 				&& (NUMBER(PG_hwpoison) != NOT_FOUND_NUMBER))
-
-static inline int
-isAnon(unsigned long mapping)
-{
-	return ((unsigned long)mapping & PAGE_MAPPING_ANON) != 0;
-}
+#define isSlab(flags)		test_bit(NUMBER(PG_slab), flags)
+#define isAnon(mapping, flags)	(((unsigned long)mapping & PAGE_MAPPING_ANON) != 0 \
+				&& !isSlab(flags))
 
 #define PTOB(X)			(((unsigned long long)(X)) << PAGESHIFT())
 #define BTOP(X)			(((unsigned long long)(X)) >> PAGESHIFT())
@@ -501,7 +498,7 @@ do { \
 #define KVER_MIN_SHIFT 16
 #define KERNEL_VERSION(x,y,z) (((x) << KVER_MAJ_SHIFT) | ((y) << KVER_MIN_SHIFT) | (z))
 #define OLDEST_VERSION		KERNEL_VERSION(2, 6, 15) /* linux-2.6.15 */
-#define LATEST_VERSION		KERNEL_VERSION(5, 15, 0) /* linux-5.15.0 */
+#define LATEST_VERSION		KERNEL_VERSION(6, 3, 0) /* linux-6.3.0 */
 
 /*
  * vmcoreinfo in /proc/vmcore
@@ -965,8 +962,25 @@ typedef unsigned long pgd_t;
 
 #ifdef __mips64__ /* mips64 */
 #define KVBASE			PAGE_OFFSET
+
+#ifndef _XKPHYS_START_ADDR
+#define _XKPHYS_START_ADDR	0x9000000000000000ULL /* _LOONGSON_XKPHYS_START_ADDR */
+#endif
+
+#ifndef _XKPHYS_END_ADDR
+#define _XKPHYS_END_ADDR	0xc000000000000000ULL /* _LOONGSON_XKPHYS_END_ADDR */
+#endif
+
+#ifndef _PAGE_OFFSET
+#define _PAGE_OFFSET		0x9800000000000000ULL
+#endif
+
 #define _SECTION_SIZE_BITS	(28)
+
+#ifndef _MAX_PHYSMEM_BITS
 #define _MAX_PHYSMEM_BITS	(48)
+#endif
+
 #define _PAGE_PRESENT		(1 << 0)
 #define _PAGE_HUGE		(1 << 4)
 
@@ -996,6 +1010,42 @@ typedef unsigned long pgd_t;
 
 #endif		/* mips64 */
 
+#ifdef __loongarch64__
+#define KVBASE			(0x8000000000000000ULL)
+#define _PAGE_OFFSET		(0x9000000000000000ULL)
+#define _XKPRANGE		(0x8000000000000000ULL)
+#define _XKVRANGE		(0xc000000000000000ULL)
+#define _SECTION_SIZE_BITS	(29)
+#define _MAX_PHYSMEM_BITS	(48)
+#define _PAGE_HUGE		(1 << 6) /* HUGE is a PMD bit */
+#define _PAGE_PRESENT		(1 << 7)
+
+typedef unsigned long pte_t;
+typedef unsigned long pmd_t;
+typedef unsigned long pgd_t;
+
+#define PAGE_MASK		(~(PAGESIZE() - 1))
+#define PMD_MASK		(~(PMD_SIZE - 1))
+#define PMD_SHIFT		((PAGESHIFT() - 3) * 2 + 3)
+#define PMD_SIZE		(1UL << PMD_SHIFT)
+#define PGDIR_SHIFT		((PAGESHIFT() - 3) * 3 + 3)
+#define PTRS_PER_PTE		(1 << (PAGESHIFT() - 3))
+#define PTRS_PER_PMD		PTRS_PER_PTE
+#define PTRS_PER_PGD		PTRS_PER_PTE
+
+#define pte_index(vaddr)		(((vaddr) >> PAGESHIFT()) & (PTRS_PER_PTE - 1))
+#define pmd_page_paddr(pmd)		(pmd & (int32_t)PAGE_MASK)
+#define pte_offset(dir, vaddr)		((pte_t *)pmd_page_paddr((*dir)) + pte_index(vaddr))
+
+#define pmd_index(vaddr)		(((vaddr) >> PMD_SHIFT) & (PTRS_PER_PMD - 1))
+#define pgd_page_paddr(pgd)		(pgd & (int32_t)PAGE_MASK)
+#define pmd_offset(pgd, vaddr)		((pmd_t *)pgd_page_paddr((*pgd)) + pmd_index(vaddr))
+
+#define pgd_index(vaddr)		(((vaddr) >> PGDIR_SHIFT) & (PTRS_PER_PGD - 1))
+#define pgd_offset(pgdir, vaddr)	((pgd_t *)(pgdir) + pgd_index(vaddr))
+
+#endif /* loongarch64 */
+
 /*
  * The function of dependence on machine
  */
@@ -1167,6 +1217,22 @@ unsigned long long vaddr_to_paddr_mips64(unsigned long vaddr);
 #define arch_crashkernel_mem_size()	stub_false()
 #endif		/* mips64 */
 
+#ifdef __loongarch64__ /* loongarch64 */
+int get_phys_base_loongarch64(void);
+int get_machdep_info_loongarch64(void);
+int get_versiondep_info_loongarch64(void);
+unsigned long long vaddr_to_paddr_loongarch64(unsigned long vaddr);
+#define find_vmemmap()		stub_false()
+#define get_phys_base()		get_phys_base_loongarch64()
+#define get_machdep_info()	get_machdep_info_loongarch64()
+#define get_versiondep_info()	get_versiondep_info_loongarch64()
+#define get_kaslr_offset(X)	stub_false()
+#define vaddr_to_paddr(X)	vaddr_to_paddr_loongarch64(X)
+#define paddr_to_vaddr(X)	paddr_to_vaddr_general(X)
+#define is_phys_addr(X)		stub_true_ul(X)
+#define arch_crashkernel_mem_size()	stub_false()
+#endif /* loongarch64 */
+
 typedef unsigned long long mdf_pfn_t;
 
 #ifndef ARCH_PFN_OFFSET
@@ -1401,6 +1467,12 @@ struct DumpInfo {
 	struct ppc64_vmemmap	*vmemmap_list;
 	unsigned long	kaslr_offset;
 
+	/*
+	 * for compound page
+	 */
+	unsigned int	compound_order_offset;
+	unsigned int	compound_dtor_offset;
+
 	/*
 	 * page table info for ppc64
 	 */
@@ -1806,6 +1878,10 @@ struct offset_table {
 		long	compound_order;
 		long	compound_head;
 	} page;
+	struct folio {
+		long	_folio_dtor;
+		long	_folio_order;
+	} folio;
 	struct mem_section {
 		long	section_mem_map;
 	} mem_section;
@@ -2301,6 +2377,12 @@ int get_xen_info_ia64(void);
 #define get_xen_info_arch(X) FALSE
 #endif	/* mips64 */
 
+#ifdef __loongarch64__ /* loongarch64 */
+#define kvtop_xen(X)	FALSE
+#define get_xen_basic_info_arch(X) FALSE
+#define get_xen_info_arch(X) FALSE
+#endif /* loongarch64 */
+
 struct cycle {
 	mdf_pfn_t start_pfn;
 	mdf_pfn_t end_pfn;
